#include <stdexcept>
#include "utils.hpp"
#include "SDL_image.h"
#include "data/hills.hpp"


std::vector<SDL_Surface *> get_hills() {
    std::vector<SDL_Surface *> surfaces;

    for (int i = 0; i < hills_length; i++) {
        HillData png = hills[i];
        SDL_RWops *rw = SDL_RWFromMem(const_cast<unsigned char *>(png.data), png.length);
        if (!rw)
            throw std::runtime_error("Failed to create RWops from memory");

        SDL_Surface *surface = IMG_Load_RW(rw, 1);
        if (!surface) {
            SDL_RWclose(rw);
            throw std::runtime_error("Failed to load image from memory: " + std::string(IMG_GetError()));
        }
        surfaces.push_back(surface);
    }

    return surfaces;
}

SDL_Surface *resize_surface(SDL_Surface *t_surface, int t_width, int t_height) {
    if (!t_surface)
        throw std::runtime_error("Original surface is null.");

    if (t_surface->w == t_width && t_surface->h == t_height)
        return t_surface;

    SDL_Surface *resizedSurface = SDL_CreateRGBSurface(
            0,
            t_width,
            t_height,
            t_surface->format->BitsPerPixel,
            t_surface->format->Rmask,
            t_surface->format->Gmask,
            t_surface->format->Bmask,
            t_surface->format->Amask
    );

    if (!resizedSurface)
        throw std::runtime_error("Failed to create resized surface: " + std::string(SDL_GetError()));

    SDL_Rect src_rect, dest_rect;
    for (int y = 0; y < t_height; ++y) {
        for (int x = 0; x < t_width; ++x) {
            src_rect.x = x * t_surface->w / t_width;
            src_rect.y = y * t_surface->h / t_height;
            src_rect.w = 1;
            src_rect.h = 1;

            dest_rect.x = x;
            dest_rect.y = y;
            dest_rect.w = 1;
            dest_rect.h = 1;

            SDL_BlitSurface(t_surface, &src_rect, resizedSurface, &dest_rect);
        }
    }

    return resizedSurface;
}


std::vector<SDL_Surface *> get_resized(const std::vector<SDL_Surface *> &t_originals, int t_width, int t_height) {
    std::vector<SDL_Surface *> ret;
    ret.reserve(t_originals.size());
    for (SDL_Surface *original: t_originals) {
        SDL_Surface *resized = resize_surface(original, t_width, t_height);
        SDL_FreeSurface(original);
        ret.push_back(resized);
    }
    return ret;
}
